
-- DS2 Database Build Scripts
-- Dave Jaffe  Todd Muirhead 8/31/05
-- Copyright Dell Inc. 2010

-- User

SET TERMOUT OFF
DROP USER DS2 CASCADE;
SET TERMOUT ON

CREATE USER DS2
  IDENTIFIED BY ds2 
  TEMPORARY TABLESPACE "TEMP"
  DEFAULT TABLESPACE "DS_MISC"
  ;

GRANT CONNECT, RESOURCE TO "DS2";


-- Tables

CREATE TABLE "DS2"."CUSTOMERS"
  (
  "CUSTOMERID" NUMBER NOT NULL, 
  "FIRSTNAME" VARCHAR2(50 byte) NOT NULL, 
  "LASTNAME" VARCHAR2(50 byte) NOT NULL, 
  "ADDRESS1" VARCHAR2(50 byte) NOT NULL, 
  "ADDRESS2" VARCHAR2(50 byte), 
  "CITY" VARCHAR2(50 byte) NOT NULL, 
  "STATE" VARCHAR2(50 byte), 
  "ZIP" NUMBER, 
  "COUNTRY" VARCHAR2(50 byte) NOT NULL, 
  "REGION" NUMBER NOT NULL,
  "EMAIL" VARCHAR2(50 byte),
  "PHONE" VARCHAR2(50 byte),
  "CREDITCARDTYPE" NUMBER NOT NULL,
  "CREDITCARD" VARCHAR2(50 byte), 
  "CREDITCARDEXPIRATION" VARCHAR2(50 byte) NOT NULL, 
  "USERNAME" VARCHAR2(50 byte) NOT NULL, 
  "PASSWORD" VARCHAR2(50 byte) NOT NULL, 
  "AGE" NUMBER, 
  "INCOME" NUMBER,
  "GENDER" VARCHAR2(6 byte)
  )
  TABLESPACE "CUSTTBS"
  PARTITION BY RANGE (REGION)
    (
    PARTITION US_PART VALUES LESS THAN (2),
    PARTITION ROW_PART VALUES LESS THAN (3),
    PARTITION MAXVAL VALUES LESS THAN (MAXVALUE)
    )
  ;

CREATE TABLE "DS2"."CUST_HIST"
  (
  "CUSTOMERID" NUMBER NOT NULL, 
  "ORDERID" NUMBER NOT NULL, 
  "PROD_ID" NUMBER NOT NULL 
  )
  TABLESPACE "CUSTTBS";
  
CREATE TABLE "DS2"."ORDERS"
  (
  "ORDERID" NUMBER NOT NULL, 
  "ORDERDATE" DATE NOT NULL, 
  "CUSTOMERID" NUMBER, 
  "NETAMOUNT" NUMBER(12, 2) NOT NULL, 
  "TAX" NUMBER(12, 2) NOT NULL, 
  "TOTALAMOUNT" NUMBER(12, 2) NOT NULL
  ) 
  TABLESPACE "ORDERTBS"
  PARTITION BY RANGE (ORDERDATE)
    (
    PARTITION JAN2009 VALUES LESS THAN (TO_DATE('2009/02/01', 'YYYY/MM/DD')),
    PARTITION FEB2009 VALUES LESS THAN (TO_DATE('2009/03/01', 'YYYY/MM/DD')),
    PARTITION MAR2009 VALUES LESS THAN (TO_DATE('2009/04/01', 'YYYY/MM/DD')),
    PARTITION APR2009 VALUES LESS THAN (TO_DATE('2009/05/01', 'YYYY/MM/DD')),
    PARTITION MAY2009 VALUES LESS THAN (TO_DATE('2009/06/01', 'YYYY/MM/DD')),
    PARTITION JUN2009 VALUES LESS THAN (TO_DATE('2009/07/01', 'YYYY/MM/DD')),
    PARTITION JUL2009 VALUES LESS THAN (TO_DATE('2009/08/01', 'YYYY/MM/DD')),
    PARTITION AUG2009 VALUES LESS THAN (TO_DATE('2009/09/01', 'YYYY/MM/DD')),
    PARTITION SEP2009 VALUES LESS THAN (TO_DATE('2009/10/01', 'YYYY/MM/DD')),
    PARTITION OCT2009 VALUES LESS THAN (TO_DATE('2009/11/01', 'YYYY/MM/DD')),
    PARTITION NOV2009 VALUES LESS THAN (TO_DATE('2009/12/01', 'YYYY/MM/DD')),
    PARTITION DEC2009 VALUES LESS THAN (TO_DATE('2010/01/01', 'YYYY/MM/DD')),
    PARTITION JAN2010 VALUES LESS THAN (TO_DATE('2010/02/01', 'YYYY/MM/DD')),
    PARTITION FEB2010 VALUES LESS THAN (TO_DATE('2010/03/01', 'YYYY/MM/DD')),
    PARTITION MAR2010 VALUES LESS THAN (TO_DATE('2010/04/01', 'YYYY/MM/DD')),
    PARTITION APR2010 VALUES LESS THAN (TO_DATE('2010/05/01', 'YYYY/MM/DD')),
    PARTITION MAY2010 VALUES LESS THAN (TO_DATE('2010/06/01', 'YYYY/MM/DD')),
    PARTITION JUN2010 VALUES LESS THAN (TO_DATE('2010/07/01', 'YYYY/MM/DD')),
    PARTITION JUL2010 VALUES LESS THAN (TO_DATE('2010/08/01', 'YYYY/MM/DD')),
    PARTITION AUG2010 VALUES LESS THAN (TO_DATE('2010/09/01', 'YYYY/MM/DD')),
    PARTITION SEP2010 VALUES LESS THAN (TO_DATE('2010/10/01', 'YYYY/MM/DD')),
    PARTITION OCT2010 VALUES LESS THAN (TO_DATE('2010/11/01', 'YYYY/MM/DD')),
    PARTITION NOV2010 VALUES LESS THAN (TO_DATE('2010/12/01', 'YYYY/MM/DD')),
    PARTITION DEC2010 VALUES LESS THAN (TO_DATE('2011/01/01', 'YYYY/MM/DD')),
    PARTITION JAN2011 VALUES LESS THAN (TO_DATE('2011/02/01', 'YYYY/MM/DD')),
    PARTITION MAXVAL VALUES LESS THAN (MAXVALUE)
    )
  ;

CREATE TABLE "DS2"."ORDERLINES"
  (
  "ORDERLINEID" NUMBER NOT NULL, 
  "ORDERID" NUMBER NOT NULL, 
  "PROD_ID" NUMBER NOT NULL, 
  "QUANTITY" NUMBER NOT NULL, 
  "ORDERDATE" DATE NOT NULL
  ) 
  TABLESPACE "ORDERTBS"
  PARTITION BY RANGE (ORDERDATE)
    (
    PARTITION JAN2009 VALUES LESS THAN (TO_DATE('2009/02/01', 'YYYY/MM/DD')),
    PARTITION FEB2009 VALUES LESS THAN (TO_DATE('2009/03/01', 'YYYY/MM/DD')),
    PARTITION MAR2009 VALUES LESS THAN (TO_DATE('2009/04/01', 'YYYY/MM/DD')),
    PARTITION APR2009 VALUES LESS THAN (TO_DATE('2009/05/01', 'YYYY/MM/DD')),
    PARTITION MAY2009 VALUES LESS THAN (TO_DATE('2009/06/01', 'YYYY/MM/DD')),
    PARTITION JUN2009 VALUES LESS THAN (TO_DATE('2009/07/01', 'YYYY/MM/DD')),
    PARTITION JUL2009 VALUES LESS THAN (TO_DATE('2009/08/01', 'YYYY/MM/DD')),
    PARTITION AUG2009 VALUES LESS THAN (TO_DATE('2009/09/01', 'YYYY/MM/DD')),
    PARTITION SEP2009 VALUES LESS THAN (TO_DATE('2009/10/01', 'YYYY/MM/DD')),
    PARTITION OCT2009 VALUES LESS THAN (TO_DATE('2009/11/01', 'YYYY/MM/DD')),
    PARTITION NOV2009 VALUES LESS THAN (TO_DATE('2009/12/01', 'YYYY/MM/DD')),
    PARTITION DEC2009 VALUES LESS THAN (TO_DATE('2010/01/01', 'YYYY/MM/DD')),
    PARTITION JAN2010 VALUES LESS THAN (TO_DATE('2010/02/01', 'YYYY/MM/DD')),
    PARTITION FEB2010 VALUES LESS THAN (TO_DATE('2010/03/01', 'YYYY/MM/DD')),
    PARTITION MAR2010 VALUES LESS THAN (TO_DATE('2010/04/01', 'YYYY/MM/DD')),
    PARTITION APR2010 VALUES LESS THAN (TO_DATE('2010/05/01', 'YYYY/MM/DD')),
    PARTITION MAY2010 VALUES LESS THAN (TO_DATE('2010/06/01', 'YYYY/MM/DD')),
    PARTITION JUN2010 VALUES LESS THAN (TO_DATE('2010/07/01', 'YYYY/MM/DD')),
    PARTITION JUL2010 VALUES LESS THAN (TO_DATE('2010/08/01', 'YYYY/MM/DD')),
    PARTITION AUG2010 VALUES LESS THAN (TO_DATE('2010/09/01', 'YYYY/MM/DD')),
    PARTITION SEP2010 VALUES LESS THAN (TO_DATE('2010/10/01', 'YYYY/MM/DD')),
    PARTITION OCT2010 VALUES LESS THAN (TO_DATE('2010/11/01', 'YYYY/MM/DD')),
    PARTITION NOV2010 VALUES LESS THAN (TO_DATE('2010/12/01', 'YYYY/MM/DD')),
    PARTITION DEC2010 VALUES LESS THAN (TO_DATE('2011/01/01', 'YYYY/MM/DD')),
    PARTITION JAN2011 VALUES LESS THAN (TO_DATE('2011/02/01', 'YYYY/MM/DD')),
    PARTITION MAXVAL VALUES LESS THAN (MAXVALUE)
    )
  ;


CREATE TABLE "DS2"."PRODUCTS"
  (
  "PROD_ID" NUMBER NOT NULL, 
  "CATEGORY" NUMBER NOT NULL, 
  "TITLE" VARCHAR2(50 byte) NOT NULL, 
  "ACTOR" VARCHAR2(50 byte) NOT NULL, 
  "PRICE" NUMBER(12, 2) NOT NULL, 
  "SPECIAL" NUMBER NOT NULL,
  "COMMON_PROD_ID" NUMBER NOT NULL
  )
  ;

CREATE TABLE "DS2"."INVENTORY"
  (
  "PROD_ID" NUMBER NOT NULL,
  "QUAN_IN_STOCK" NUMBER NOT NULL,
  "SALES" NUMBER NOT NULL
  )
  ;

CREATE TABLE "DS2"."CATEGORIES"
  (
  "CATEGORY" NUMBER NOT NULL, 
  "CATEGORYNAME" VARCHAR2(50 byte) NOT NULL, 
  CONSTRAINT "PK_CATEGORIES" PRIMARY KEY("CATEGORY")
  )
  ;

  INSERT INTO "DS2"."CATEGORIES" (CATEGORY, CATEGORYNAME) VALUES (1,'Action');
  INSERT INTO "DS2"."CATEGORIES" (CATEGORY, CATEGORYNAME) VALUES (2,'Animation');
  INSERT INTO "DS2"."CATEGORIES" (CATEGORY, CATEGORYNAME) VALUES (3,'Children');
  INSERT INTO "DS2"."CATEGORIES" (CATEGORY, CATEGORYNAME) VALUES (4,'Classics');
  INSERT INTO "DS2"."CATEGORIES" (CATEGORY, CATEGORYNAME) VALUES (5,'Comedy');
  INSERT INTO "DS2"."CATEGORIES" (CATEGORY, CATEGORYNAME) VALUES (6,'Documentary');
  INSERT INTO "DS2"."CATEGORIES" (CATEGORY, CATEGORYNAME) VALUES (7,'Drama');
  INSERT INTO "DS2"."CATEGORIES" (CATEGORY, CATEGORYNAME) VALUES (8,'Family');
  INSERT INTO "DS2"."CATEGORIES" (CATEGORY, CATEGORYNAME) VALUES (9,'Foreign');
  INSERT INTO "DS2"."CATEGORIES" (CATEGORY, CATEGORYNAME) VALUES (10,'Games');
  INSERT INTO "DS2"."CATEGORIES" (CATEGORY, CATEGORYNAME) VALUES (11,'Horror');
  INSERT INTO "DS2"."CATEGORIES" (CATEGORY, CATEGORYNAME) VALUES (12,'Music');
  INSERT INTO "DS2"."CATEGORIES" (CATEGORY, CATEGORYNAME) VALUES (13,'New');
  INSERT INTO "DS2"."CATEGORIES" (CATEGORY, CATEGORYNAME) VALUES (14,'Sci-Fi');
  INSERT INTO "DS2"."CATEGORIES" (CATEGORY, CATEGORYNAME) VALUES (15,'Sports');
  INSERT INTO "DS2"."CATEGORIES" (CATEGORY, CATEGORYNAME) VALUES (16,'Travel');


CREATE TABLE "DS2"."REORDER" 
  (
  "PROD_ID" NUMBER NOT NULL, 
  "DATE_LOW" DATE NOT NULL, 
  "QUAN_LOW" NUMBER NOT NULL, 
  "DATE_REORDERED" DATE, 
  "QUAN_REORDERED" NUMBER, 
  "DATE_EXPECTED" DATE
  )  
  ;


-- Sequences

CREATE SEQUENCE "DS2"."CUSTOMERID_SEQ" 
  INCREMENT BY 1 
--START WITH 20001 
  START WITH 2000001
--START WITH 200000001
  MAXVALUE 1.0E28 
  MINVALUE 1 
  NOCYCLE 
  CACHE 100000 
  NOORDER
  ;

CREATE SEQUENCE "DS2"."ORDERID_SEQ" 
  INCREMENT BY 1 
--START WITH 20001
  START WITH 2000001
--START WITH 200000001 
  MAXVALUE 1.0E28 
  MINVALUE 1 
  NOCYCLE 
  CACHE 100000 
  NOORDER
  ;


-- Data types

CREATE OR REPLACE  PACKAGE "DS2"."DS2_TYPES"  AS
  TYPE DS2_CURSOR IS REF CURSOR;
  TYPE ARRAY_TYPE IS TABLE OF VARCHAR2(50) INDEX BY BINARY_INTEGER;
  TYPE N_TYPE IS TABLE OF NUMBER INDEX BY BINARY_INTEGER;
END DS2_TYPES;
/

COMMIT;

EXIT;
